<?php
/**
 * @package Freestyle Joomla
 * @author Freestyle Joomla
 * @copyright (C) 2013 Freestyle Joomla
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
**/
defined('_JEXEC') or die;
class FSJ_NoticesCheck {
	static function ShouldDisplay($notice)
	{
		if (!self::Check_Publish($notice))
			return false;
		if (!self::Check_Time($notice))
			return false;
		if (!self::Check_DOW($notice))
			return false;
		if (!self::Check_DayOfMonth($notice))
			return false;
		if (!self::Check_Month($notice))
			return false;
		if (!self::Check_Group($notice))
			return false;
		if (!self::Check_Access($notice))
			return false;
		if (!self::Check_Languages($notice))
			return false;
		if (!self::Check_Menu($notice))
			return false;
		if (!self::Check_Component($notice))
			return false;
		// 	
		return true;	
	}
	static function Check_Publish($notice)
	{
		$curtime = time();
		$pubup = strtotime($notice->publish_up);
		$pubdown = strtotime($notice->publish_down);
		if ($pubdown < 0)
			$pubdown = 0;
		if ($pubup > $curtime)
		{
			$notice->reason = "Time is before publish up time - Pub Up Time: " . date("Y-m-d H:i:s", $pubup) . ", Cur Time: " . date("Y-m-d H:i:s", $curtime);
			return false;
		}
		if ($pubdown && $pubdown < $curtime)
		{
			$notice->reason = "Time is after publish down time - Pub Down Time: " . date("Y-m-d H:i:s", $pubdown) . ", Cur Time: " . date("Y-m-d H:i:s", $curtime);
			return false;
		}
		return true;
	}
	static function Check_Menu($notice)
	{
		if (!isset($notice->settings->notices_menu->menuitem_use))
			return true;
		if (!isset($notice->settings->notices_menu->menuitem))
			$notice->settings->notices_menu->menuitem = "";
		$items = explode(";", $notice->settings->notices_menu->menuitem);
		$currentitem = JRequest::getVar('Itemid',0);
		if (in_array($currentitem, $items))
		{
			$show = (bool)$notice->settings->notices_menu->menuitem_use;	
		} else {
			$show = !(bool)$notice->settings->notices_menu->menuitem_use;
		}
		if (!$show)
			$notice->reason = "Menu Item didnt match";
		return $show;
	}
	static function Check_Component($notice)
	{
		if (!isset($notice->settings->notices_menu->component_use))
			return true;
		if (!isset($notice->settings->notices_menu->component))
			$notice->settings->notices_menu->component = "";
		$items = explode(";", $notice->settings->notices_menu->component);
		$currentitem = JRequest::getVar('option','');
		if (in_array($currentitem, $items))
		{
			$show = (bool)$notice->settings->notices_menu->component_use;	
		} else {
			$show = !(bool)$notice->settings->notices_menu->component_use;
		}
		if (!$show)
			$notice->reason = "Component didnt match";
		return $show;
	}
	static function Check_Time($notice)
	{
		if (!isset($notice->settings->notices_data_time->time_use))
			return true;
		$current = time();
		$inclusive = (bool)$notice->settings->notices_data_time->time_use;	
		$timefrom = strtotime($notice->settings->notices_data_time->time_from);
		$timeto = strtotime($notice->settings->notices_data_time->time_to);
		if ($timeto < $timefrom)
		{
			$inclusive = !$inclusive;
		}
		if ($timefrom > $timeto)
		{
			$temp = $timeto;
			$timeto = $timefrom;
			$timefrom = $temp;	
		}
		if ($current >= $timefrom && $current <= $timeto)
		{
			$show = $inclusive;
		} else {
			$show = !$inclusive;
		}
		if (!$show)
		{
			$ct = date("H:i", $current);
			$st = date("H:i", $timefrom);
			$et = date("H:i", $timeto);
			$i = ($inclusive) ? "not between" : "between";
			$notice->reason = "Didnt match time - Current time $ct is $i $st and $et";
		}
		return $show;
	}
	static function Check_DOW($notice)
	{
		if (!isset($notice->settings->notices_data_time->dayofweek_use))
			return true;
		if (!isset($notice->settings->notices_data_time->dayofweek))
			$notice->settings->notices_data_time->dayofweek = "";
		$days = explode(";", $notice->settings->notices_data_time->dayofweek);
		$currentdow = date("w", time());
		$show = false;
		if (in_array($currentdow, $days))
			$show = true;	
		$notice->reason = "Day of week didnt match";
		if (isset($notice->settings->notices_data_time->dayofweek_monthpos))
		{
			$ok = false;
			$day_names = array(1 => 'Mon', 2 => 'Tue', 3 => 'Wed', 4 => 'Thu', 5 => 'Fri', 6 => 'Sat', 7 => 'Sun');
			$which = "";
			switch ($notice->settings->notices_data_time->dayofweek_monthpos)
			{
				case 1:
					$which = "first";
					break;
				case 2:
					$which = "second";	
					break;
				case 3:
					$which = "third";
					break;
				case 4;
					$which = "fourth";
					break;
				case 5:
					$which = "fifth";
					break;
				case 99:
					$which = "last";
					break;	
			}
			$today = date("Y-m-d");
			foreach ($days as $day)
			{
				$dayname = $day_names[$day];
				$lookup = "$which $dayname of this month";
				$time = strtotime($lookup);
				$date = date("Y-m-d", $time);
				if ($today == $date)//echo "Today : $today, Checking $date<br>";
					$ok = true;
			}
			if (!$ok)
			{
				$show = false;
				$notice->reason = "Day of week in month didnt match";
			}
		}
		if (!(bool)$notice->settings->notices_data_time->dayofweek_use)
			$show = !$show;
		if ($show)
			$notice->reason = "";
		return $show;				
	}
	static function Check_DayOfMonth($notice)
	{
		if (!isset($notice->settings->notices_data_time->dayinmonth_use))
			return true;		
		if (!isset($notice->settings->notices_data_time->dayinmonth))
			$notice->settings->notices_data_time->dayinmonth = "";
		$dates = explode(";", $notice->settings->notices_data_time->dayinmonth);
		$currentdate = date("d", time());
		if (in_array($currentdate, $dates))
		{
			$show = (bool)$notice->settings->notices_data_time->dayinmonth_use;	
		} else {
			$show = !(bool)$notice->settings->notices_data_time->dayinmonth_use;
		}
		if (!$show)
			$notice->reason = "Day in month didnt match";
		return $show;			
	}
	static function Check_Month($notice)
	{
		if (!isset($notice->settings->notices_data_time->month_use))
			return true;
		if (!isset($notice->settings->notices_data_time->month))
			$notice->settings->notices_data_time->month = "";
		$months = explode(";", $notice->settings->notices_data_time->month);
		$currentmonth = date("n", time());
		if (in_array($currentmonth, $months))
		{
			$show = (bool)$notice->settings->notices_data_time->month_use;	
		} else {
			$show = !(bool)$notice->settings->notices_data_time->month_use;
		}
		if (!$show)
			$notice->reason = "Month didnt match";
		return $show;			
	}
	static function Check_Group($notice)
	{
		if (!isset($notice->settings->notices_user->group_use))
			return true;
		if (!isset($notice->settings->notices_user->group))
			$notice->settings->notices_user->group = "";
		$user = JFactory::getUser();		
		$user_groups = $user->getAuthorisedGroups();
		$check_groups = explode(";", $notice->settings->notices_user->group);
		if ($notice->settings->notices_user->group_use) // include groups
		{
			foreach ($check_groups as $group)
			{
				if (in_array($group, $user_groups))
					return true;
			}
			$notice->reason = "The user is not in any of the specified groups";
			return false;
		} else {
			foreach ($check_groups as $group)
			{
				if (in_array($group, $user_groups))
				{
					$notice->reason = "The user is in an excluded group";
					return false;
				}
			}
			return true;
		}
	}
	static function Check_Access($notice)
	{
		if (!isset($notice->settings->notices_user->accesslevel_use))
			return true;
		if (!isset($notice->settings->notices_user->accesslevel))
			$notice->settings->notices_user->accesslevel = "";
		$user = JFactory::getUser();		
		$user_access = $user->getAuthorisedViewLevels();
		$check_access = explode(";", $notice->settings->notices_user->accesslevel);
		if ($notice->settings->notices_user->accesslevel_use) // include groups
		{
			foreach ($check_access as $group)
			{
				if (in_array($group, $user_access))
					return true;
			}
			$notice->reason = "The user is not in any of the specified access levels";
			return false;
		} else {
			foreach ($check_access as $group)
			{
				if (in_array($group, $user_access))
				{
					$notice->reason = "The user is in an excluded access level";
					return false;
				}
			}
			return true;
		}	
	}	
	static function Check_Languages($notice)
	{
		if (!isset($notice->settings->notices_misc->language_use))
			return true;
		if (!isset($notice->settings->notices_misc->language))
			$notice->settings->notices_misc->language = "";
		$lang = JFactory::getLanguage();
		$cur_lang = $lang->getTag();
		$check_lang = explode(";", $notice->settings->notices_misc->language);
		if (in_array($cur_lang, $check_lang))
		{
			$show = (bool)$notice->settings->notices_misc->language_use;	
		} else {
			$show = !(bool)$notice->settings->notices_misc->language_use;
		}
		if (!$show)
			$notice->reason = "Language didnt match";
		return $show;			
	}
	// 
	static function SplitList($in)
	{
		$funcs = $in;
		$funcs = str_replace(",","\n",$funcs);
		$funcs = str_replace(";","\n",$funcs);
		$funcs = explode("\n", $funcs);
		$func_array = array();
		foreach ($funcs as $func)
		{
			$func = trim($func);
			if ($func)
				$func_array[] = $func;	
		}
		return $func_array;
	}
}
