<?php
/**
 * @package Freestyle Joomla
 * @author Freestyle Joomla
 * @copyright (C) 2013 Freestyle Joomla
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
**/
defined('_JEXEC') or die;

class FSJ_Installer
{
	function Install($data)
	{
		$xml = simplexml_load_file($data->xml_file);
		//echo "Load {$data->xml_file}<br>";
		$type = $xml->attributes()->type;

		$name = (string)$xml->description;
		// need to determine if we are a new install or an upgrade
		$class = "FSJ_Install_$type";
		if (!class_exists($class))
		{
			require_once($data->installerdir . DS . "{$type}.php");
		}
		
		$post = new $class();
		$post->PreInstall($data, $installer, $xml);	
		
		$type_disp = ucfirst($type);
		
		echo "<tr>";

		if ($type == "plugin")
			$type_disp .= " - " . ucfirst($xml->attributes()->group);

		echo "<td>$type_disp</td>";
		
		echo "<td>$name</td>";
		
		$xml = simplexml_load_file($data->xml_file);
		if ($xml)
		{
			echo "<td>" . FSJ_FormatVersion($xml->version) . "</td>";
		} else {
			echo "<td></td>";	
		}
		
		// need to check if there is an existing version of the component available
		if (!$post->InstallCheck($data, $installer, $xml))
		{
			echo "<td style='color: {$post->result_col}'>{$post->result}</td>";
		} else {
		
			$installer = new JInstaller();
			$installer->setOverwrite(true);
		
			if ($installer->install($data->source))
			{
				echo "<td style='color: #00c000'>OK</td>";
			
				$post->PostInstall($data, $installer, $xml);
				
			} else {
				echo "<td style='color: #c00000'>Failed</td>";	
			}
		}
		
		
		echo "</tr>";
	}
}

function FSJ_FormatVersion($version)
{
	$parts = explode(".", $version);
	$ver = "{$parts[0]}.{$parts[1]}.{$parts[2]}";
	
	if (isset($parts[3]))
		$ver .= " (build {$parts[3]})";
	
	return $ver;
}

class FSJ_Install_Type
{
	var $result_col = '';
	var $result = '';
	
	function PostInstall(&$data, &$installer, &$xml)
	{
		
	}	
	
	function PreInstall(&$data, &$installer, &$xml)
	{
		
	}	
	
	function InstallCheck(&$data, &$installer, &$xml)
	{
		if (!$this->VersionCheck($data, $installer, $xml))
			return false;
		
		if (!$this->ProLiteCheck($data, $installer, $xml))
			return false;
		
		return true;	
	}
	
	function VersionCheck(&$data, &$installer, &$xml)
	{
		// check if there is a newer version install, if so then dont install
		$new_xml = simplexml_load_file($data->xml_file);
		
		$name = (string)$new_xml->name;
		
		$db = JFactory::getDBO();
		$qry = "SELECT * FROM #__extensions WHERE name = '" . $db->escape($name) . "'";
		$db->setQuery($qry);
		$item = $db->loadObject();
		
		if (!$item)
			return true;
		
		$manifest = json_decode($item->manifest_cache);
		
		$existing_version = $manifest->version;
		
		$new_version = (string)$new_xml->version;
		
		//echo "Old : $existing_version<br>";
		//echo "New: $new_version<br>";
		
		if ($existing_version == "1.0.0")
			return true;
		
		if (version_compare($existing_version, $new_version) == 1)
		{
			$this->result = "Newer version already installed, v" . FSJ_FormatVersion($existing_version);
			$this->result_col = "#0000c0";
			return false;
		}
		return true;
	}
	
	function ProLiteCheck(&$data, &$installer, &$xml)
	{
		// check if there is a pro version installed and we are installing a lite one then dont install
		
		return true;
	}
	
	function GetExistingXML($data)
	{
		return null;	
	}
}